/* ----------------------------------------------------------------------------
 *  codingSpace18					
 * ----------------------------------------------------------------------------
 *  GUI - Slider
 * ----------------------------------------------------------------------------
 *  prog: eddi.lecesne@zhdk.ch
 *  13.11.2010
 * ----------------------------------------------------------------------------
 */

class Slider
{
  PVector _p1;
  PVector _p2;
  float[] _a = new float[4];

  int     _w;
  int     _h;
  float   _pos;



  Slider(int x,int y,int h,int w,float pos)
  {   
    _p1 = new PVector(x + w * 0.5, y);
    _p2 = new PVector(x + w *.5, y + h);

    _w = w;
    _h = h;

    _pos = pos;
    if(_pos > 1.0)
      _pos = 1.0;
    else if(_pos < 0.0)
      _pos = 0.0;
  }

  float pos()
  {
    return _pos;
  }
  //fill color
  void fillColor(float c1, float c2, float c3, float a) {
    _a[0] = c1;
    _a[1] = c2;
    _a[2] = c3;
    _a[3] = a;
  }

  boolean isHit(int x,int y)
  {
    if(x > _p1.x - 20 && x < _p1.x + 20 && y > _p1.y  && y < _p2.y) {

      return true;
    }
    else {
      return false;
    }
  }


  void draw()
  {
    pushStyle();
    drawSlider();
    drawKnob();
    popStyle();
  } 

  void drawSlider()
  {
    noFill();
    stroke(255, 255,255, 255);
    smooth();
    strokeWeight(2);   

    bezier(_p1.x-(_w/2.0), _p1.y, _p1.x-(_w/10.0), _p1.y + (_h/8.0),  _p2.x-(_w/10.0), _p2.y-(_h/8.0), _p2.x-(_w/2.0), _p2.y);  //left side
    bezier(_p1.x+(_w/2.0), _p1.y, _p1.x+(_w/10.0), _p1.y + (_h/8.0), _p2.x +(_w/10.0), _p2.y-(_h/8.0), _p2.x+(_w/2.0), _p2.y);  //right side
    bezier(_p1.x-(_w/2.0), _p1.y, _p1.x-(_w/5.0), _p1.y-(_h/12), _p1.x+(_w/5.0), _p1.y-(_h/12), _p1.x+(_w/2.0), _p1.y);  //top
    bezier(_p2.x-(_w/2.0), _p2.y, _p2.x-(_w/5.0), _p2.y+(_h/12), _p2.x+(_w/5.0), _p2.y+(_h/12), _p2.x+(_w/2.0), _p2.y);  //bottom
  }

  void drawKnob()
  {
    stroke(255, 255,255, 255);
    
    PVector dir = PVector.sub(_p2,_p1);
    PVector pos = PVector.add( _p1, PVector.mult(dir,_pos));
    noFill();
    float bezier1= pos.x-(_w/10.0);
    float bezier2= pos.x+(_w/10.0);
    float bezier3= pos.x-(_w/10.0);
    float bezier4= pos.x+(_w/10.0);
    float bezier5= pos.y+(_h/20.0);
    float bezier6= pos.y-(_h/20.0);

    if(_pos >= 0.6) {
      bezier1 = map(_pos, 0.5, 1.0, pos.x-(_w/10.0), pos.x-(_w/3.0) );  //change bottom left side
      bezier2 = map(_pos, 0.5, 1.0, pos.x+(_w/10.0), pos.x+(_w/3.0) );  //change bottom right side
      bezier6 = map(_pos, 0.6, 0.0, pos.y+(_h/10.0), pos.y+(_h/3.0) );  //expand top middle
    }
    else
      bezier6= pos.y-(_h/20.0);
      
    if(_pos <= 0.6) {
      bezier3 = map(_pos, 0.5, 0.0, pos.x-(_w/10.0), pos.x-(_w/3.0));  //top left side
      bezier4 = map(_pos, 0.5, 0.0, pos.x+(_w/10.0), pos.x+(_w/3.0));  ///top right side
      bezier5 = map(_pos, 0.6, 1.0, pos.y-(_h/50.0), pos.y-(_h/3.0) );  //expand bottom middle
    }
    else
       bezier5= pos.y+(_h/20.0);

    rect(pos.x-3, pos.y-2, 6, 4);
    bezier(bezier3, pos.y-4, pos.x-(_w/15.0), bezier6, pos.x+(_w/15.0), bezier6, bezier4, pos.y-4);  //top arc
    bezier(bezier1, pos.y+4, pos.x-(_w/15.0), bezier5, pos.x+(_w/15.0),bezier5, bezier2, pos.y+4);  //bottom arc

    bezier(bezier3, pos.y-4, pos.x-(_w/10.0), pos.y-2, pos.x-(_w/10.0), pos.y+2, bezier1, pos.y+4);  //left side arc
    bezier(bezier4, pos.y-4, pos.x+(_w/10.0), pos.y-2, pos.x+(_w/10.0), pos.y+2, bezier2, pos.y+4);  //right side arc
  }

  void mousePos(int x,int y,boolean pressed)
  {
    if(pressed)
    {
      if(isHit(x,y)) {
        // move knob
        PVector dir = PVector.sub(_p1, _p2);  //laenge der 2 punkte bestimmen
        _pos = 1.0 / dir.mag() * (y-_p1.y);  //mapping im zahlenbereich (laenge in scalar) . mag (magnitude) berechnet die laenge in scalar (eine zahl)
      }
    }
    else
    {  // mouse over
    }
  }
  void mouseOver(int x, int  y) {
    if(isHit(x,y)) {
      pushStyle();
      noFill();
      int t = 10;
      strokeWeight(t  );
      
      
      //fade
      float al=100;
      al-= (millis()%1000/10.0);

      stroke(255, 255, 255, 100);
      
      //as drawn at the beginning (can be different)
      bezier(_p1.x-(_w/2.0), _p1.y, _p1.x-(_w/10.0), _p1.y + (_h/8.0),  _p2.x-(_w/10.0), _p2.y-(_h/8.0), _p2.x-(_w/2.0), _p2.y);  //left side
      bezier(_p1.x+(_w/2.0), _p1.y, _p1.x+(_w/10.0), _p1.y + (_h/8.0), _p2.x +(_w/10.0), _p2.y-(_h/8.0), _p2.x+(_w/2.0), _p2.y);  //right side
      bezier(_p1.x-(_w/2.0), _p1.y, _p1.x-(_w/5.0), _p1.y-(_h/12), _p1.x+(_w/5.0), _p1.y-(_h/12), _p1.x+(_w/2.0), _p1.y);  //top
      bezier(_p2.x-(_w/2.0), _p2.y, _p2.x-(_w/5.0), _p2.y+(_h/12), _p2.x+(_w/5.0), _p2.y+(_h/12), _p2.x+(_w/2.0), _p2.y);  //bottom
      popStyle();
      
    }
  }
}

